"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sh = require("shelljs");
const debugTools_1 = require("../helpers/debugTools");
const TslintRunner_1 = require("../tslint/TslintRunner");
const TSC_COMPILATION_COMPLETE = /(?:Compilation complete\.|Found \d+ errors?\.) Watching for file changes/;
const TSC_COMPILATION_STARTED = /File change detected. Starting incremental compilation|Starting compilation in watch mode.../;
exports.TSC_ERRORS_FOUND = /^([^\s].*)\((\d+|\d+,\d+|\d+,\d+,\d+,\d+)\):\s+(error)\s+(TS\d+)\s*:\s*(.*)$/gm;
class Project {
    constructor(args, compilingCb, compiledCb) {
        this.resultBuffer = [];
        this.lastResult = "";
        this.tslintRunner = null;
        this.parseCommandOutput = (data) => {
            if (!this.resultBuffer)
                this.resultBuffer = [];
            if (data.match(TSC_COMPILATION_COMPLETE)) {
                debugTools_1.debugLog("Tsc: complete and printing everything for", this.args.path);
                this.setLastResult();
                this.compiledCb(this);
                if (this.tslintRunner) {
                    this.compilingCb();
                    this.tslintRunner.startLinting();
                }
            }
            else if (data.match(TSC_COMPILATION_STARTED)) {
                // Push empty result for old tsc compatibility
                this.resultBuffer.push("");
                debugTools_1.debugLog("Tsc: starting compilation for", this.args.path);
                if (this.tslintRunner)
                    this.tslintRunner.terminate();
                this.compilingCb();
            }
            else if (data) {
                // Show 'Starting compilation in watch mode...' for old tsc clients
                if (!this.resultBuffer.length) {
                    this.compilingCb();
                    debugTools_1.debugLog("Tsc: starting compilation for", this.args.path);
                }
                this.resultBuffer.push(data);
            }
        };
        this.tsLintDoneCb = () => {
            this.compiledCb(this);
        };
        this.args = args;
        this.compilingCb = compilingCb;
        this.compiledCb = compiledCb;
        if (this.args.tslint !== undefined) {
            this.tslintRunner = new TslintRunner_1.TslintRunner(this.args.tslint, this.tsLintDoneCb);
        }
    }
    startCompiling() {
        const { compiler, watch, path, noEmit } = this.args;
        const compileCommand = [
            compiler,
            watch ? "-w" : "",
            "--pretty false",
            "--preserveWatchOutput",
            noEmit === true ? "--noEmit" : "",
            `-p ${path}`
        ]
            .filter(value => !!value)
            .join(" ");
        debugTools_1.debugLog("Tsc: executing following command", compileCommand);
        const execOptions = { async: true, silent: true };
        const child = sh.exec(compileCommand, execOptions);
        child.stdout.on("data", this.parseCommandOutput);
        child.stdout.on("end", () => {
            debugTools_1.debugLog("Tsc: done with compiling for", this.args.path);
            this.setLastResult();
            if (this.tslintRunner)
                this.tslintRunner.startLinting();
            else
                this.compiledCb(this);
        });
    }
    setLastResult() {
        this.lastResult = this.resultBuffer.join("");
        this.resultBuffer = null;
    }
    getLastResult() {
        const result = [];
        if (this.lastResult)
            result.push(this.lastResult);
        if (this.tslintRunner) {
            const tslintResult = this.tslintRunner.getLastResult();
            if (tslintResult)
                result.push(tslintResult);
        }
        return result.join("\n");
    }
    isCompiling() {
        return !!this.resultBuffer || (this.tslintRunner ? this.tslintRunner.isRunning() : false);
    }
    equals(otherProjectPath) {
        return this.args.path.toLocaleLowerCase() === otherProjectPath.toLocaleLowerCase();
    }
}
exports.Project = Project;
