"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const moment = require("moment");
const debugTools_1 = require("../helpers/debugTools");
const Project_1 = require("./Project");
class ProjectsWatcher {
    constructor(watch) {
        this.watch = watch;
        this.finalOutput = "";
        this.lastLog = "IDLE";
        this.projects = [];
        this.projectCompilationStart = () => {
            if (this.lastLog === "IDLE" || this.lastLog === "COMPLETE") {
                this.logStatus("COMPILING");
            }
        };
        this.projectCompilationComplete = (project) => {
            if (!this.watch) {
                this.projectCompilationFinal(project);
                return;
            }
            const result = this.projects
                .map(projectCompiler => {
                return projectCompiler.getLastResult();
            })
                .join("\n");
            console.log(result);
            this.logStatus("COMPLETE");
            if (this.isAProjectCompiling()) {
                this.logStatus("COMPILING");
            }
        };
    }
    addProject(projectArgs) {
        if (this.findProject(projectArgs.path)) {
            debugTools_1.debugLog("Ignored duplicate project", projectArgs.path);
            return;
        }
        debugTools_1.debugLog("Creating project compiler for", projectArgs.path);
        const newProjectCompiler = new Project_1.Project(projectArgs, this.projectCompilationStart, this.projectCompilationComplete);
        this.projects.push(newProjectCompiler);
    }
    startCompilations() {
        this.projects.forEach(project => project.startCompiling());
    }
    projectCompilationFinal(project) {
        const lastResult = project.getLastResult();
        console.log(lastResult + "\n");
        this.projects.splice(this.projects.indexOf(project), 1);
        this.finalOutput += "\n" + lastResult;
        if (!this.projects.length) {
            this.logStatus("IDLE");
        }
    }
    findProject(path) {
        return this.projects.find(project => project.equals(path)) || null;
    }
    getTimestamp() {
        return moment().format("HH:mm:ss");
    }
    isAProjectCompiling() {
        return this.projects.find(project => project.isCompiling());
    }
    logStatus(type) {
        let message;
        if (type === "COMPILING" && this.lastLog === "IDLE") {
            message = "Starting compilation in watch mode...";
        }
        else if (type === "COMPILING") {
            message = "File change detected. Starting incremental compilation...";
        }
        else if (type === "COMPLETE") {
            message = "Compilation complete. Watching for file changes.";
        }
        else {
            const foundErrors = this.finalOutput.match(Project_1.TSC_ERRORS_FOUND);
            if (foundErrors) {
                message = `Done compiling all projects with ${foundErrors.length} error(s) (see above).`;
                console.log(`${this.getTimestamp()} - ${message}`);
                process.exit(1);
            }
            message = `Done compiling all projects without errors.`;
        }
        console.log(`${this.getTimestamp()} - ${message}`);
        this.lastLog = type;
    }
}
exports.ProjectsWatcher = ProjectsWatcher;
